@@ -32,26 +32,6 @@
 #include "panel-profile.h"
 #include "panel-shell.h"
 
-void
-panel_session_request_logout (void)
-{
-	GnomeClient *client;
-
-	if (!(client = gnome_master_client ()))
-		return;
-
-	/* Only request a Global save. We only want a Local
-	 * save if the user selects 'Save current setup'
-	 * from the dialog.
-	 */
-	gnome_client_request_save (client,
-				   GNOME_SAVE_GLOBAL,
-				   TRUE,
-				   GNOME_INTERACT_ANY,
-				   FALSE,
-				   TRUE);
-}
-
 static void
 panel_session_handle_die_request (GnomeClient *client)
 {
@@ -69,11 +49,26 @@ panel_session_handle_die_request (GnomeClient *client)
 	gtk_main_quit ();
 }
 
+void
+panel_session_do_not_restart (void)
+{
+	GnomeClient *client;
+
+	client = gnome_master_client ();
+
+	gnome_client_set_restart_style (client, GNOME_RESTART_IF_RUNNING);
+}
+
 void
 panel_session_init (void)
 {
 	GnomeClient *client;
 
+	/* We don't want the WM to try and save/restore our
+	 * window position
+	 */
+	gdk_set_sm_client_id (NULL);
+
 	client = gnome_master_client ();
 
         if (!getenv ("GNOME_PANEL_DEBUG"))
